<?php
namespace page;
// +----------------------------------------------------------------------
// | ThinkPHP [ WE CAN DO IT JUST THINK ]
// +----------------------------------------------------------------------
// | Copyright (c) 2006~2017 http://thinkphp.cn All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | 优化版分页插件
// +----------------------------------------------------------------------
// | Author: 美奇软件开发工作室 QQ：15577969
// +----------------------------------------------------------------------

use think\Paginator;

class Page extends Paginator
{

    //首页
    protected function home() {
        if ($this->currentPage() > 1) {
            return "<li class='page-item'><a class='page-link' href='" . $this->url(1) . "'>首页</a></li>";
        } else {
            return "<li class='page-item disabled'><a class='page-link'>首页</a></li>";
        }
    }

    //上一页
    protected function prev() {
        if ($this->currentPage() > 1) {
            return "<li class='page-item'><a class='page-link' href='" . $this->url($this->currentPage - 1) . "'>上一页</a></li>";
        } else {
            return "<li class='page-item disabled'><a class='page-link'>上一页</a></li>";
        }
    }

    //下一页
    protected function next() {
        if ($this->hasMore) {
            return "<li class='page-item'><a class='page-link' href='" . $this->url($this->currentPage + 1) . "'>下一页</a></li>";
        } else {
            return "<li class='page-item disabled'><a class='page-link'>下一页</a></li>";
        }
    }

    //尾页
    protected function last() {
        if ($this->hasMore) {
            return "<li class='page-item'><a class='page-link' href='" . $this->url($this->lastPage) . "'>尾页</a></li>";
        } else {
            return "<li class='page-item disabled'><a class='page-link'>尾页</a></li>";
        }
    }

    //统计信息
    protected function info(){
        $html='<p class="pageRemark">总共有<b>' . $this->lastPage .'</b>页，';
        $html.='<b>'. $this->total . '</b>条数据&nbsp;&nbsp;';
        $html.='<select id="pagesize"> ';
        $html.='<option value="5" selected="">5 条/页</option> ';
        $html.='<option value="10">10 条/页</option> ';
        $html.='<option value="20">20 条/页</option> ';
        $html.='<option value="30">30 条/页</option> ';
        $html.='<option value="50">50 条/页</option> ';
        $html.='</select> ';
        $html.='&nbsp;&nbsp;到第<input id="pageInput" type="text" min="1" value="1" class="page-input">页 ';
        $html.='<button type="button" href="javascript:void(0)" onclick="gotoPage()" class="page-btn">确定</button> ';
        $html.='</p>';
        return $html;
    }

    /**
     * 页码按钮
     * @return string
     */
    protected function getLinks()
    {

        $block = [
            'first'  => null,
            'slider' => null,
            'last'   => null
        ];

        $side   = 3;
        $window = $side * 2;

        if ($this->lastPage < $window + 6) {
            $block['first'] = $this->getUrlRange(1, $this->lastPage);
        } elseif ($this->currentPage <= $window) {
            $block['first'] = $this->getUrlRange(1, $window + 2);
            $block['last']  = $this->getUrlRange($this->lastPage - 1, $this->lastPage);
        } elseif ($this->currentPage > ($this->lastPage - $window)) {
            $block['first'] = $this->getUrlRange(1, 2);
            $block['last']  = $this->getUrlRange($this->lastPage - ($window + 2), $this->lastPage);
        } else {
            $block['first']  = $this->getUrlRange(1, 2);
            $block['slider'] = $this->getUrlRange($this->currentPage - $side, $this->currentPage + $side);
            $block['last']   = $this->getUrlRange($this->lastPage - 1, $this->lastPage);
        }

        $html = '';

        if (is_array($block['first'])) {
            $html .= $this->getUrlLinks($block['first']);
        }

        if (is_array($block['slider'])) {
            $html .= $this->getDots();
            $html .= $this->getUrlLinks($block['slider']);
        }

        if (is_array($block['last'])) {
            $html .= $this->getDots();
            $html .= $this->getUrlLinks($block['last']);
        }

        return $html;
    }

    /**
     * 渲染分页html
     * @return mixed
     */
    public function render()
    {
        if ($this->hasPages()) {
            if ($this->simple) {
                return sprintf(
                    '<ul class="pagination">%s %s %s</ul>',
                    $this->prev(),
                    $this->getLinks(),
                    $this->next()
                );
            } else {
                return sprintf(
                    '<ul class="pagination">%s %s %s %s %s</ul>',
                    $this->home(),
                    $this->prev(),
                    $this->getLinks(),
                    $this->next(),
                    $this->last()
                );
            }
        }
    }

    /**
     * 生成一个可点击的按钮
     *
     * @param  string $url
     * @param  int    $page
     * @return string
     */
    protected function getAvailablePageWrapper($url, $page)
    {
        return "<li class='page-item'><a class='page-link' href='" . htmlentities($url) . "'>{$page}</a></li>";
    }

    /**
     * 生成一个禁用的按钮
     *
     * @param  string $text
     * @return string
     */
    protected function getDisabledTextWrapper($text)
    {
        return "<li class='page-item disabled'><a class='page-link'>{$text}</a></li>";
    }

    /**
     * 生成一个激活的按钮
     *
     * @param  string $text
     * @return string
     */
    protected function getActivePageWrapper($text)
    {
        return '<li class="page-item active" aria-current="page"><a class="page-link" href="#">'.$text.'</a></li>';
    }

    /**
     * 生成省略号按钮
     *
     * @return string
     */
    protected function getDots()
    {
        return $this->getDisabledTextWrapper('...');
    }

    /**
     * 批量生成页码按钮.
     *
     * @param  array $urls
     * @return string
     */
    protected function getUrlLinks(array $urls)
    {
        $html = '';

        foreach ($urls as $page => $url) {
            $html .= $this->getPageLinkWrapper($url, $page);
        }

        return $html;
    }

    /**
     * 生成普通页码按钮
     *
     * @param  string $url
     * @param  int    $page
     * @return string
     */
    protected function getPageLinkWrapper($url, $page)
    {
        if ($page == $this->currentPage()) {
            return $this->getActivePageWrapper($page);
        }

        return $this->getAvailablePageWrapper($url, $page);
    }
}